local LT = {};
local DisplayTimes;

LT.version = "0.1";

function LT:HideLevelFrames()
  for key, realm in pairs (DisplayTimes.times) do
    for key, player in pairs (realm) do
      for key, level in pairs (player) do
        level:Hide();
      end
    end
  end
end

function LT:UpdateDisplay()
  LT:HideLevelFrames();
  LT.FilteredList = {};
  LT.TotalFiltered = 0;
  LT.TotalTime = 0;
  for level, times in pairs(LevelTimer[LT.Realm][LT.Player]) do
    if LevelTimer[LT.Realm][LT.Player][level]["end"] ~= nil then
      if level >= LT.FilterBegin and level <= LT.FilterEnd then
        tinsert(LT.FilteredList, level);
      end
    end
  end
  table.sort(LT.FilteredList);
  LT.TotalFiltered = #LT.FilteredList;
  DisplayTimes.levels:SetSize(190, LT.TotalFiltered * 15);

  local topMost = false;
  for key, level in pairs(LT.FilteredList) do
    LT.TotalTime = LT.TotalTime + LevelTimer[LT.Realm][LT.Player][level]["total"];
    DisplayTimes.times[LT.Realm][LT.Player][level]:Show();
    if not topMost then
      DisplayTimes.times[LT.Realm][LT.Player][level]:SetPoint("TOPLEFT", DisplayTimes.levels, "TOPLEFT", 0, 0);
      topMost = true;
    else
      DisplayTimes.times[LT.Realm][LT.Player][level]:SetPoint("TOPLEFT", DisplayTimes.times[LT.Realm][LT.Player][level-1], "TOPLEFT", 0, -15);
    end
  end
  DisplayTimes.totalTxt:SetText(LT:parseTimeLong(LT.TotalTime));

  if UIDropDownMenu_GetText(DisplayTimes.endDropDown) == nil and #LT.LevelList > 0 then
    UIDropDownMenu_SetText(DisplayTimes.endDropDown, math.max(unpack(LT.LevelList))+1);
  end
  if UIDropDownMenu_GetText(DisplayTimes.beginDropDown) == nil and #LT.LevelList > 0 then
    UIDropDownMenu_SetText(DisplayTimes.beginDropDown, math.min(unpack(LT.LevelList)));
  end

end

function LT:highlightFrame(frame)
  frame:SetColorTexture(0.2, 0.2, 0.2, 0.5);
end

function LT:unhighlightFrame(frame)
  frame:SetColorTexture(0.2, 0.2, 0.2, 0);
end

function LT.BuildLevelList()
  LT.LevelList = {};
  local player = LevelTimer[GetRealmName()][LT.Player];
  for level, times in pairs(player) do
    if player[level]["end"] ~= nil then
      tinsert(LT.LevelList, level);
    end
  end
  LT.TotalLevels = #LT.LevelList;
  if LT.TotalLevels == 0 then
    return
  end
  table.sort(LT.LevelList);
  LT.FilterBegin = math.min(unpack(LT.LevelList));
  LT.FilterEnd = math.max(unpack(LT.LevelList));
end

function LT:AddToBlacklist(player)
  tinsert(LevelTimer[LT.Realm]["LT_BLACKLIST"], player);
  LT:UpdatePlayerFrames();
end

function LT:ConfirmDelete(player)
  StaticPopupDialogs["LEVELTIMER_CONFIRMDEL"] = {
    text = "Are you sure you want to delete data for |cffff0000" .. player .. "|r?",
    button1 = "Yes",
    button2 = "No",
    OnAccept = function()
      LT:AddToBlacklist(player);
    end,
    timeout = 0,
    whileDead = true,
    hideOnEscape = true,
    preferredIndex = 3,
  }
  StaticPopup_Show ("LEVELTIMER_CONFIRMDEL");
end

function LT:PlayerClicked(frame, button)
  if button == "RightButton" and frame:GetText() ~= GetUnitName("player") then
    LT:ConfirmDelete(frame:GetText());
    return;
  end
  
  for player, playerFrames in pairs (DisplayTimes.PlayerNames) do
    DisplayTimes.PlayerNames[player].text:SetTextColor(1, 1, 1, 1);
  end
  frame:SetTextColor(0, 1, 0, 1);
  LT.Player = frame:GetText();
  LT:BuildLevelList();
  if #LT.LevelList > 0 then
    UIDropDownMenu_SetText(DisplayTimes.beginDropDown, LT.FilterBegin);
    UIDropDownMenu_SetText(DisplayTimes.endDropDown, LT.FilterEnd+1);
  else
    UIDropDownMenu_SetText(DisplayTimes.beginDropDown, nil);
    UIDropDownMenu_SetText(DisplayTimes.endDropDown, nil);
  end
  LT:UpdateDisplay();
end

local function addLevelTime(point, relativeFrame, relativePoint, xOffset, yOffset, text)
  local txt = childLevels:CreateFontString(nil, "OVERLAY");
  txt:SetFontObject("GameFontHighlight");
  txt:SetPoint(point, relativeFrame, relativePoint, xOffset, yOffset);
  txt:SetText(text);
  return txt;
end

function LT:parseTimeShort(seconds)
  local seconds = tonumber(seconds);
  hours = string.format("%01.f", math.floor(seconds/3600));
  mins = string.format("%01.f", math.floor(seconds/60 - (hours*60)));
  secs = string.format("%01.f", math.floor(seconds - (hours*3600) - (mins *60)));
  local timeStr;
  if tonumber(hours) < 1 then
    if tonumber(mins) < 1 then
      timeStr = secs .. "s";
    else
      timeStr = mins .. "m " .. secs .. "s";
    end
  else
    timeStr = hours .."h " .. mins .. "m " .. secs .. "s";
  end
  return timeStr;
end

function LT:parseTimeLong(seconds)
  if seconds == 0 then
    return "No data logged for this character";
  end
  local seconds = tonumber(seconds);
  hours = string.format("%01.f", math.floor(seconds/3600));
  mins = string.format("%01.f", math.floor(seconds/60 - (hours*60)));
  secs = string.format("%01.f", math.floor(seconds - (hours*3600) - (mins *60)));
  local timeStr;
  if tonumber(hours) < 1 then
    if tonumber(mins) < 1 then
      timeStr = secs .. " seconds";
    else
      timeStr = mins .. " minutes, " .. secs .. " seconds";
    end
  else
    timeStr = hours .." hours, " .. mins .. " minutes, " .. secs .. " seconds";
  end
  return timeStr;
end

function LT:CreatePlayerFrames()
  local playerNum = 0;
  DisplayTimes.PlayerNames = {};
  for key, player in pairs (LT.PlayerList) do 
    DisplayTimes.PlayerNames[player] = {};
    DisplayTimes.PlayerNames[player] = CreateFrame("Frame", nil, DisplayTimes.players);
    DisplayTimes.PlayerNames[player]:SetSize(94, 15);
    DisplayTimes.PlayerNames[player].highlight = DisplayTimes.PlayerNames[player]:CreateTexture(nil, "HIGHLIGHT");
    DisplayTimes.PlayerNames[player].highlight:SetAllPoints(true);

    DisplayTimes.PlayerNames[player]:SetPoint("TOPLEFT", DisplayTimes.players, "TOPLEFT", 0, playerNum * -15);
    DisplayTimes.PlayerNames[player]:EnableMouse(true);

    DisplayTimes.PlayerNames[player].text = DisplayTimes.PlayerNames[player]:CreateFontString(nil, "OVERLAY");
    DisplayTimes.PlayerNames[player].text:SetFontObject("GameFontHighlight");
    DisplayTimes.PlayerNames[player].text:SetPoint("TOPLEFT", DisplayTimes.PlayerNames[player] , "TOPLEFT", 2, -1);
    DisplayTimes.PlayerNames[player].text:SetText(player);

    DisplayTimes.PlayerNames[player]:SetScript("OnMouseDown", function(self, button) LT:PlayerClicked(DisplayTimes.PlayerNames[player].text, button); end);
    if player == GetUnitName("player") then
      DisplayTimes.PlayerNames[player].text:SetTextColor(0, 1, 0, 1);
    end
    DisplayTimes.PlayerNames[player]:Hide();
    playerNum = playerNum + 1;
  end
end

function LT:UpdatePlayerFrames()
  local playerNum = 0;
  for key, player in pairs (LT.PlayerList) do
    DisplayTimes.PlayerNames[player]:Hide();
    if player ~= "LT_BLACKLIST" and not tContains(LevelTimer[GetRealmName()]["LT_BLACKLIST"], player) then
      DisplayTimes.PlayerNames[player]:Show();
      DisplayTimes.PlayerNames[player]:SetPoint("TOPLEFT", DisplayTimes.players, "TOPLEFT", 0, playerNum * -15);
      playerNum = playerNum + 1;
    end
  end
end

function LT:CreateTimeFrames()
  DisplayTimes.times = {};
  for realm, realms in pairs (LevelTimer) do
    DisplayTimes.times[realm] = {};
    for name, characters in pairs (realms) do
      DisplayTimes.times[realm][name] = {};
      for level, levels in pairs (characters) do
        if LevelTimer[realm][name][level]["end"] ~= nil then
          DisplayTimes.times[realm][name][level] = {};
          DisplayTimes.times[realm][name][level] = CreateFrame("Frame", nil, DisplayTimes.levels);
          DisplayTimes.times[realm][name][level]:SetSize(190, 15);
          DisplayTimes.times[realm][name][level].highlight = DisplayTimes.times[realm][name][level]:CreateTexture(nil, "HIGHLIGHT");
          DisplayTimes.times[realm][name][level].highlight:SetAllPoints(true);

          DisplayTimes.times[realm][name][level]:SetPoint("TOPLEFT", DisplayTimes.levels, "TOPLEFT", 0, 0);
          DisplayTimes.times[realm][name][level]:EnableMouse(true);
          DisplayTimes.times[realm][name][level]:SetScript("OnEnter", function() LT:highlightFrame(DisplayTimes.times[realm][name][level].highlight) end);
          DisplayTimes.times[realm][name][level]:SetScript("OnLeave", function() LT:unhighlightFrame(DisplayTimes.times[realm][name][level].highlight) end);

          local txt = DisplayTimes.times[realm][name][level]:CreateFontString(nil, "OVERLAY");
          txt:SetFontObject("GameFontHighlight");
          txt:SetPoint("TOPLEFT", DisplayTimes.times[realm][name][level] , "TOPLEFT", 2, -1);
          txt:SetText(level+1);

          txt = DisplayTimes.times[realm][name][level]:CreateFontString(nil, "OVERLAY");
          txt:SetFontObject("GameFontHighlight");
          txt:SetPoint("TOPRIGHT", DisplayTimes.times[realm][name][level], "TOPRIGHT", -2, -1);
          txt:SetText(LT:parseTimeShort(LevelTimer[realm][name][level]["total"]));
        end
      end
    end
  end
  LT:UpdateDisplay();
end

function LT:CreateTimeFrameOnNewLevel(level)
  level = level - 1;
  DisplayTimes.times[GetRealmName()][GetUnitName("player")][level] = {};
  DisplayTimes.times[GetRealmName()][GetUnitName("player")][level] = CreateFrame("Frame", nil, DisplayTimes.levels);
  DisplayTimes.times[GetRealmName()][GetUnitName("player")][level]:SetSize(190, 15);
  DisplayTimes.times[GetRealmName()][GetUnitName("player")][level].highlight = DisplayTimes.times[GetRealmName()][GetUnitName("player")][level]:CreateTexture(nil, "HIGHLIGHT");
  DisplayTimes.times[GetRealmName()][GetUnitName("player")][level].highlight:SetAllPoints(true);

  DisplayTimes.times[GetRealmName()][GetUnitName("player")][level]:SetPoint("TOPLEFT", DisplayTimes.levels, "TOPLEFT", 0, 0);
  DisplayTimes.times[GetRealmName()][GetUnitName("player")][level]:EnableMouse(true);
  DisplayTimes.times[GetRealmName()][GetUnitName("player")][level]:SetScript("OnEnter", function() LT:highlightFrame(DisplayTimes.times[GetRealmName()][GetUnitName("player")][level].highlight) end);
  DisplayTimes.times[GetRealmName()][GetUnitName("player")][level]:SetScript("OnLeave", function() LT:unhighlightFrame(DisplayTimes.times[GetRealmName()][GetUnitName("player")][level].highlight) end);

  local txt = DisplayTimes.times[GetRealmName()][GetUnitName("player")][level]:CreateFontString(nil, "OVERLAY");
  txt:SetFontObject("GameFontHighlight");
  txt:SetPoint("TOPLEFT", DisplayTimes.times[GetRealmName()][GetUnitName("player")][level] , "TOPLEFT", 2, -1);
  txt:SetText(level+1);

  txt = DisplayTimes.times[GetRealmName()][GetUnitName("player")][level]:CreateFontString(nil, "OVERLAY");
  txt:SetFontObject("GameFontHighlight");
  txt:SetPoint("TOPRIGHT", DisplayTimes.times[GetRealmName()][GetUnitName("player")][level] , "TOPRIGHT", -2, -1);
  txt:SetText(LT:parseTimeShort(LevelTimer[GetRealmName()][GetUnitName("player")][level]["total"]));

  LT.BuildLevelList();
  if #LT.LevelList > 0 then
    UIDropDownMenu_SetText(DisplayTimes.beginDropDown, LT.FilterBegin);
    UIDropDownMenu_SetText(DisplayTimes.endDropDown, LT.FilterEnd+1);
  else
    UIDropDownMenu_SetText(DisplayTimes.beginDropDown, nil);
    UIDropDownMenu_SetText(DisplayTimes.endDropDown, nil);
  end
  LT:UpdateDisplay();
  LT:PlayerClicked(DisplayTimes.PlayerNames[GetUnitName("player")].text);
end


local function setBegin(self, level)
  LT.FilterBegin = level;
  UIDropDownMenu_SetText(DisplayTimes.beginDropDown, LT.FilterBegin);
  if LT.FilterBegin >= LT.FilterEnd then
    UIDropDownMenu_SetText(DisplayTimes.endDropDown, LT.FilterBegin + 1);
    LT.FilterEnd = LT.FilterBegin;
  end
  LT:UpdateDisplay();
end

local function setEnd(self, level)
  LT.FilterEnd = level;
  UIDropDownMenu_SetText(DisplayTimes.endDropDown, LT.FilterEnd+1);
  if LT.FilterEnd <= LT.FilterBegin then
    UIDropDownMenu_SetText(DisplayTimes.beginDropDown, LT.FilterEnd);
    LT.FilterBegin = level;
  end
  LT:UpdateDisplay();
end

LT.Events = {};
LT.Events.Leveling = CreateFrame("Frame");
LT.Events.Login = CreateFrame("Frame");
LT.Events.Leveled = CreateFrame("Frame");
LT.Events.Begin = CreateFrame("Frame");

function LT.BuildPlayerList()
  LT.PlayerList = {};
  for player, players in pairs (LevelTimer[GetRealmName()]) do
    tinsert(LT.PlayerList, player);
  end
  table.sort(LT.PlayerList);
  LT.TotalPlayers = #LT.PlayerList;
end

local function DropDown_AddLevelsBegin()
  local info = UIDropDownMenu_CreateInfo();
  info.func = setBegin;
  for key, level in pairs(LT.LevelList) do
    info.text, info.arg1 = level, level;
    info.notCheckable = true;
    UIDropDownMenu_AddButton(info);
  end
end

local function DropDown_AddLevelsEnd()
  local info = UIDropDownMenu_CreateInfo();
  info.func = setEnd;
  for key, level in pairs(LT.LevelList) do
    info.text, info.arg1 = level+1, level;
    info.notCheckable = true;
    UIDropDownMenu_AddButton(info);
  end
end

local function PlayerLogin()
  LT.Realm = GetRealmName();
  LT.Player = GetUnitName("player");
  if type(LevelTimer) ~= "table" then LevelTimer = {}; end
  if type(LevelTimer[LT.Realm]) ~= "table" then LevelTimer[LT.Realm] = {}; end
  if type(LevelTimer[LT.Realm][LT.Player]) ~= "table" then LevelTimer[LT.Realm][LT.Player] = {}; end
  if type(LevelTimer[LT.Realm]["LT_BLACKLIST"]) ~= "table" then LevelTimer[LT.Realm]["LT_BLACKLIST"] = {}; end
  LT.Me = LevelTimer[LT.Realm][LT.Player];
  LT.BuildLevelList();
  LT.BuildPlayerList();
  LT.ShowTimes = DisplayTimes or LT:CreateDisplay();
  LT.ShowTimes:Hide();
end

LT.Events.Login:SetScript('OnEvent', PlayerLogin);
LT.Events.Login:RegisterEvent("PLAYER_LOGIN");

function LT:CreateText(point, relativeFrame, relativePoint, xOffset, yOffset, text)
  local txt = DisplayTimes:CreateFontString(nil, "OVERLAY");
  txt:SetFontObject("GameFontHighlight");
  txt:SetPoint(point, relativeFrame, relativePoint, xOffset, yOffset);
  txt:SetText(text);
  return txt;
end

function LT:CreateDisplay()

  DisplayTimes = CreateFrame("Frame", "LevelTimerFrame", UIParent, "UIPanelDialogTemplate");
  tinsert(UISpecialFrames, DisplayTimes:GetName());
  DisplayTimes:SetSize(350,460);
  DisplayTimes:SetPoint("CENTER", UIParent, "CENTER");

  DisplayTimes:SetMovable(true);
  DisplayTimes:EnableMouse(true);
  DisplayTimes:RegisterForDrag("LeftButton");
  DisplayTimes:SetScript("OnDragStart", DisplayTimes.StartMoving);
  DisplayTimes:SetScript("OnDragStop", DisplayTimes.StopMovingOrSizing);

  DisplayTimes.title = DisplayTimes:CreateFontString(nil, "OVERLAY");
  DisplayTimes.title:SetFontObject("GameFontHighlight");
  DisplayTimes.title:SetPoint("LEFT", LevelTimerFrameTitleBG, "LEFT", 5, 0);
  DisplayTimes.title:SetText("LevelTimer v".. LT.version);

  DisplayTimes.filters = CreateFrame("Frame", nil, DisplayTimes);
  DisplayTimes.filters:SetSize(328, 40);
  DisplayTimes.filters:SetPoint("TOPLEFT", LevelTimerFrameDialogBG, "TOPLEFT", 4, -8);

  DisplayTimes.characters = CreateFrame("Frame", nil, DisplayTimes);
  DisplayTimes.characters:SetSize(117, 350);
  DisplayTimes.characters:SetPoint("TOPLEFT", DisplayTimes.filters, "TOPLEFT", 0, -42);

  DisplayTimes.total = CreateFrame("Frame", nil, DisplayTimes);
  DisplayTimes.total:SetSize(328, 25);
  DisplayTimes.total:SetPoint("BOTTOMLEFT", LevelTimerFrameDialogBG, "BOTTOMLEFT", 4, 4);

  DisplayTimes.filterBorder = CreateFrame("Frame", nil, DisplayTimes);
  DisplayTimes.filterBorder:SetSize(328, 2);
  DisplayTimes.filterBorder.bg = DisplayTimes.filterBorder:CreateTexture(nil, "BACKGROUND");
  DisplayTimes.filterBorder.bg:SetAllPoints(true);
  DisplayTimes.filterBorder:SetPoint("TOPLEFT", DisplayTimes.filters, "TOPLEFT", 0, -40);
  DisplayTimes.filterBorder:SetBackdrop({bgFile=nil, edgeFile="Interface\\BUTTONS\\WHITE8X8", edgeSize=2});
  DisplayTimes.filterBorder:SetBackdropBorderColor(0.3, 0.3, 0.3, 1);

  DisplayTimes.totalBorder = CreateFrame("Frame", nil, DisplayTimes);
  DisplayTimes.totalBorder:SetSize(328, 2);
  DisplayTimes.totalBorder.bg = DisplayTimes.totalBorder:CreateTexture(nil, "BACKGROUND");
  DisplayTimes.totalBorder.bg:SetAllPoints(true);
  DisplayTimes.totalBorder:SetPoint("TOPLEFT", DisplayTimes.total, "TOPLEFT", 0, 0);
  DisplayTimes.totalBorder:SetBackdrop({bgFile=nil, edgeFile="Interface\\BUTTONS\\WHITE8X8", edgeSize=2});
  DisplayTimes.totalBorder:SetBackdropBorderColor(0.3, 0.3, 0.3, 1);

  DisplayTimes.charactersBorder = CreateFrame("Frame", nil, DisplayTimes);
  DisplayTimes.charactersBorder:SetSize(2, 350)
  DisplayTimes.charactersBorder.bg = DisplayTimes.charactersBorder:CreateTexture(nil, "BACKGROUND");
  DisplayTimes.charactersBorder.bg:SetAllPoints(true);
  DisplayTimes.charactersBorder:SetPoint("TOPRIGHT", DisplayTimes.characters, "TOPRIGHT", 0, 0);
  DisplayTimes.charactersBorder:SetBackdrop({bgFile=nil, edgeFile="Interface\\BUTTONS\\WHITE8X8", edgeSize=2})
  DisplayTimes.charactersBorder:SetBackdropBorderColor(0.3, 0.3, 0.3, 1)

  DisplayTimes.beginDropDown = CreateFrame("Frame", nil, DisplayTimes.filters, "UIDropDownMenuTemplate");
  DisplayTimes.beginDropDown:SetPoint("TOPLEFT", DisplayTimes.filters, "TOPLEFT", -13, -10);
  DisplayTimes.beginDropDown:SetWidth("30");
  UIDropDownMenu_Initialize(DisplayTimes.beginDropDown, DropDown_AddLevelsBegin);

  DisplayTimes.endDropDown = CreateFrame("Frame", nil, DisplayTimes.filters, "UIDropDownMenuTemplate")
  DisplayTimes.endDropDown:SetPoint("TOPLEFT", DisplayTimes.filters, "TOPLEFT", 172, -10)
  DisplayTimes.endDropDown:SetWidth("30")
  UIDropDownMenu_Initialize(DisplayTimes.endDropDown, DropDown_AddLevelsEnd)

  DisplayTimes.beginLabel = LT:CreateText("TOPLEFT", DisplayTimes.filters, "TOPLEFT", 5, 0, "Begin:");
  DisplayTimes.endLabel = LT:CreateText("TOPLEFT", DisplayTimes.filters, "TOPLEFT", 190, 0, "End:");
  DisplayTimes.totalTxt = LT:CreateText("CENTER", DisplayTimes.total, "CENTER", 0, -2, "");


  DisplayTimes.ScrollFrame = CreateFrame("ScrollFrame", nil, DisplayTimes, "UIPanelScrollFrameTemplate");
  DisplayTimes.ScrollFrame:SetPoint("TOPLEFT", DisplayTimes.filterBorder, "TOPLEFT", 120, -4);
  DisplayTimes.ScrollFrame:SetPoint("BOTTOMRIGHT", DisplayTimes.totalBorder, "BOTTOMRIGHT", 0, 4);

  DisplayTimes.levels = CreateFrame("Frame", nil, DisplayTimes.ScrollFrame);
  DisplayTimes.levels:SetSize(190, LT.TotalLevels * 15);

  DisplayTimes.ScrollFrame:SetScrollChild(DisplayTimes.levels);
  DisplayTimes.ScrollFrame.ScrollBar:ClearAllPoints();
  DisplayTimes.ScrollFrame.ScrollBar:SetPoint("TOPLEFT", DisplayTimes.ScrollFrame, "TOPRIGHT", -10, -16);
  DisplayTimes.ScrollFrame.ScrollBar:SetPoint("BOTTOMRIGHT", DisplayTimes.ScrollFrame, "BOTTOMRIGHT", -7, 15);
  DisplayTimes.ScrollFrame:SetClipsChildren(true);

  DisplayTimes.PlayerScrollFrame = CreateFrame("ScrollFrame", nil, DisplayTimes, "UIPanelScrollFrameTemplate");
  DisplayTimes.PlayerScrollFrame:SetPoint("TOPLEFT", DisplayTimes, "TOPLEFT", 12, -76);
  DisplayTimes.PlayerScrollFrame:SetSize(115, 345);

  DisplayTimes.players = CreateFrame("Frame", nil, DisplayTimes.PlayerScrollFrame);
  DisplayTimes.players:SetSize(94, LT.TotalPlayers * 15);

  DisplayTimes.PlayerScrollFrame:SetScrollChild(DisplayTimes.players);
  DisplayTimes.PlayerScrollFrame.ScrollBar:ClearAllPoints();
  DisplayTimes.PlayerScrollFrame.ScrollBar:SetPoint("TOPLEFT", DisplayTimes.PlayerScrollFrame, "TOPRIGHT", -10, -16);
  DisplayTimes.PlayerScrollFrame.ScrollBar:SetPoint("BOTTOMRIGHT", DisplayTimes.PlayerScrollFrame, "BOTTOMRIGHT", -7, 15);
  DisplayTimes.PlayerScrollFrame:SetClipsChildren(true);

  LT:CreateTimeFrames();
  LT:CreatePlayerFrames();
  LT:UpdatePlayerFrames();
  LT:UpdateDisplay();
  return DisplayTimes;

end

function LT:Toggle()
  LT.ShowTimes:SetShown(not LT.ShowTimes:IsShown());
end

local function LevelTimerSlash(msg)
  local _, _, cmd, args = string.find(msg, "%s?(%w+)%s?(.*)")
  if cmd == "undel" then
    for k,BlackListedName in pairs(LevelTimer[GetRealmName()]["LT_BLACKLIST"]) do 
      if args == BlackListedName then
        table.remove(LevelTimer[GetRealmName()]["LT_BLACKLIST"], k);
        LT:UpdatePlayerFrames();
        LT:UpdateDisplay();
        print("LevelTimer: Undeleting the character |cffff0000" .. args .. "|r.");
      end
    end
  else
    LT:UpdateDisplay();
    LT:Toggle();
  end
end

SLASH_LEVELTIMER1, SLASH_LEVELTIMER2 = "/lt", "/leveltimer";
SlashCmdList["LEVELTIMER"] = LevelTimerSlash;

local function LevelUpMsg(...)
  local lvl1, lvl2;
  lvl1 = select(1, ...);
  if select(2, ...) then
    lvl2 = select(2, ...);
  end
  local str;
  if lvl2 == nil then
    str = "It took " .. LT:parseTimeLong(LT.Me[lvl1 - 1]["total"]) .. " from level " .. lvl1 - 1 .. " to level " .. lvl1 .. ".";
  else
    local totalSec = 0;
    for key, item in pairs(LT.Me) do
      if key >= lvl1 and key < lvl2 then
        totalSec = totalSec + tonumber(LT.Me[key]["total"]);
      end
    end
    str = "It took " .. LT:parseTimeLong(totalSec) .. " from level " .. lvl1 .. " to level " .. lvl2 .. ".";
  end
  return str;
end

function LT:AtMaxLevel()
  if UnitLevel("player") == MAX_PLAYER_LEVEL then
    LT.Logging = false;
    return true;
  end
  return false;
end

local function BeginLogging(self, event, timestamp, ...)
  if LT.Me[UnitLevel("player")] == nil and not LT.AtMaxLevel() then
    LT.Me[UnitLevel("player")] = {};
    LT.Me[UnitLevel("player")]["begin"] = timestamp;
    LT.Events.Begin:UnregisterEvent("TIME_PLAYED_MSG");
  end
end

local function PlayerLeveledUp(self, event, timestamp, ...)
  LT.Me[UnitLevel("player") -1]["end"] = timestamp;
  LT.Me[UnitLevel("player") -1]["total"] = LT.Me[UnitLevel("player")-1]["end"] - LT.Me[UnitLevel("player")-1]["begin"];
  if not LT:AtMaxLevel() then
    LT.Me[UnitLevel("player")] = {};
    LT.Me[UnitLevel("player")]["begin"] = timestamp;
  end
  print(LevelUpMsg(UnitLevel("player")));
  LT:CreateTimeFrameOnNewLevel(UnitLevel("player"));
  LT:UpdateDisplay();
  LT.Events.Leveled:UnregisterEvent("TIME_PLAYED_MSG");
end

do
  local requesting;
  local o = _G.ChatFrame_DisplayTimePlayed;
  _G.ChatFrame_DisplayTimePlayed = function(...)
  if requesting then
    requesting = false;
    return;
  end
  return o(...);
end
function LT:RequestTimePlayed()
  requesting = true;
  _G.RequestTimePlayed();
end
end

local function LevelXPChange(self, event, ...)
if event == "PLAYER_XP_UPDATE" then
  if not LT:AtMaxLevel() and not LT.Logging then
    LT.Logging = true;
    if LT.Me[UnitLevel("player")] == nil then
      LT.Events.Begin:RegisterEvent('TIME_PLAYED_MSG');
      LT.Events.Begin:SetScript('OnEvent', BeginLogging);
      LT:RequestTimePlayed();
      LT:UpdateDisplay();
    end
  end
end
if event == "PLAYER_LEVEL_CHANGED" then
  LT.Events.Leveled:RegisterEvent('TIME_PLAYED_MSG');
  LT.Events.Leveled:SetScript('OnEvent', PlayerLeveledUp);
  LT:RequestTimePlayed();
end
end

LT.Events.Leveling:SetScript('OnEvent', LevelXPChange);
LT.Events.Leveling:RegisterEvent("PLAYER_LEVEL_CHANGED");
LT.Events.Leveling:RegisterEvent("PLAYER_XP_UPDATE");